/**
 * @fileoverview Enforce ES5 or ES6 class for returning value in render function.
 * @author Mark Orel
 */
'use strict';

// ------------------------------------------------------------------------------
// Requirements
// ------------------------------------------------------------------------------

var rule = require('../../../lib/rules/require-render-return');
var RuleTester = require('eslint').RuleTester;

var parserOptions = {
  ecmaVersion: 6,
  ecmaFeatures: {
    jsx: true
  }
};

require('babel-eslint');

// ------------------------------------------------------------------------------
// Tests
// ------------------------------------------------------------------------------

var ruleTester = new RuleTester();
ruleTester.run('require-render-return', rule, {

  valid: [{
    // ES6 class
    code: [
      'class Hello extends React.Component {',
      '  render() {',
      '    return <div>Hello {this.props.name}</div>;',
      '  }',
      '}'
    ].join('\n'),
    parserOptions: parserOptions
  }, {
    // ES5 class
    code: [
      'var Hello = React.createClass({',
      '  displayName: \'Hello\',',
      '  render: function() {',
      '    return <div></div>',
      '  }',
      '});'
    ].join('\n'),
    parserOptions: parserOptions
  }, {
    // Return in a switch...case
    code: [
      'var Hello = React.createClass({',
      '  render: function() {',
      '    switch (this.props.name) {',
      '      case \'Foo\':',
      '        return <div>Hello Foo</div>;',
      '      default:',
      '        return <div>Hello {this.props.name}</div>;',
      '    }',
      '  }',
      '});'
    ].join('\n'),
    parserOptions: parserOptions
  }, {
    // Return in a if...else
    code: [
      'var Hello = React.createClass({',
      '  render: function() {',
      '    if (this.props.name === \'Foo\') {',
      '      return <div>Hello Foo</div>;',
      '    } else {',
      '      return <div>Hello {this.props.name}</div>;',
      '    }',
      '  }',
      '});'
    ].join('\n'),
    parserOptions: parserOptions
  }, {
    // Not a React component
    code: [
      'class Hello {',
      '  render() {}',
      '}'
    ].join('\n'),
    parserOptions: parserOptions
  }],

  invalid: [{
    // Missing return in ES5 class
    code: [
      'var Hello = React.createClass({',
      '  displayName: \'Hello\',',
      '  render: function() {}',
      '});'
    ].join('\n'),
    parserOptions: parserOptions,
    errors: [{
      message: 'Your render method should have return statement'
    }]
  }, {
    // Missing return in ES6 class
    code: [
      'class Hello extends React.Component {',
      '  render() {} ',
      '}'
    ].join('\n'),
    parserOptions: parserOptions,
    errors: [{
      message: 'Your render method should have return statement'
    }]
  }, {
    // Missing return (but one is present in a sub-function)
    code: [
      'class Hello extends React.Component {',
      '  render() {',
      '    const names = this.props.names.map(function(name) {',
      '      return <div>{name}</div>',
      '    });',
      '  } ',
      '}'
    ].join('\n'),
    parserOptions: parserOptions,
    errors: [{
      message: 'Your render method should have return statement'
    }]
  }
]});
